<?php

	/*
	*	Gekko - Open Source Web Development Framework
	*	------------------------------------------------------------------------
	*	Copyright (C) 2004-2005, J. Carlos Nieto <xiam@users.sourceforge.net>
	*	This program is Free Software.
	*
	*	@package	Gekko
	*	@license	http://www.gnu.org/copyleft/gpl.html GNU/GPL License 2.0
	*	@author		J. Carlos Nieto <xiam@users.sf.net>
	*	@version	CVS: $Id:$
	*	@link		http://gekkoware.sourceforge.net
	*/

	function addsqlslashes($text) {
		$text = str_replace("\\", "\\\\", $text);
		$text = str_replace("'", "\'", $text);
		$text = str_replace("\n", "\\n", $text);
		$text = str_replace("\r", "\\r", $text);
		$text = str_replace("\t", "\\t", $text);
		return $text;
	}

	function stripsqlslashes($text) {
		$text = str_replace("\\n", "\n", $text);
		$text = str_replace("\\r", "\r", $text);
		$text = str_replace("\\t", "\t", $text);
		$text = str_replace("\'", "'", $text);
		$text = str_replace("\\\\", "\\", $text);
		return $text;
	}

	function sql_value_format(&$v) {
		$v = "'".addsqlslashes($v)."'";
	}

	define ("IN-GEKKO", true);

	require_once "../../conf.php";
	require_once GEKKO_ROOT."lib/core.lib.php";

	db_init();

	require_once GEKKO_ROOT."lib/auth.inc.php";

	if (!gauth("backup")) abort(accessDenied());

	gpGet("action", 1);
	gpGet("id", 1);
	gpGet("return", 1);

	switch ($action) {

		case "create":

			gpGlobalize("file, opt, comp");

			// is filename specified?
			if (!$file) $file = preg_replace("/[^a-z0-9]/", "-", strtolower(_l("C_SITE.NAME")))."_backup_".date("YmdHis")."";

			// compression method
			switch ($comp) {
				case "bz2": case "gzip": case "sql":
				break;
				default:
					// checking for the best compression engine available
					if (extension_loaded("bz2")) {
						// bzip2 has a better compression ratio than zlib
						$comp = "bz2";
					} elseif (extension_loaded("zlib")) {
						$comp = "gzip";
					} else {
						$comp = "sql";
					}
				break;
			}

			$backup_start = getmicrotime();

			$buff = "";

			// drop database?
			if (isset($opt["drop_db"])) {
				$buff .= "# Dropping database \"".$db->info("name")."\"\n";
				$buff .= "DROP DATABASE IF EXISTS `".$db->info("name")."`;\n";
				$buff .= "CREATE DATABASE `".$db->info("name")."`;\n\n";
			}

			// getting tables
			$tables = $db->get_tables();

			while ($table = $db->fetch_row($tables, SQL_NUM)) {

				$table = $table[0];

				// only tables using a specified prefix
				if (($opt["table_prefix"]) && (substr($table, 0, strlen($opt["table_prefix"])) != $opt["table_prefix"])) continue;

				$buff .= "--\n";
				$buff .= "-- Table structure for table `$table`\n";
				$buff .= "--\n\n";

				if (isset($opt["drop_table"])) {
					// adding drop table
					$buff .= "DROP TABLE IF EXISTS `$table`;\n";
				}

				$q = $db->query("SHOW CREATE TABLE `$table`", __FILE__, __LINE__, true);
				$r = $db->fetch_row($q, SQL_NUM);
				$buff .= $r[1].";\n\n";

				// dumping table data
				$fields = "";
				$q = $db->query("SELECT * FROM `$table`", __FILE__, __LINE__, true);
				if ($db->num_rows($q)) {
					$buff .= "--\n";
					$buff .= "-- Dumping data for table `$table`\n";
					$buff .= "--\n\n";
					// listing table fields
					$fields = $db->get_fields($q);
					while ($row = $db->fetch_row($q, SQL_NUM)) {
						array_walk($row, "sql_value_format");
						$buff .= "INSERT INTO `{$table}` "."(".implode(", ", $fields).")"." VALUES (".implode(", ", $row).");\n";
					}
				}

				$buff .= "\n\n";
			}

			$header = "";
			$header .= "--\n";
			$header .= "-- Gekko ".GEKKO_VERSION." Database Backup (".$db->info("type").")\n";
			$header .= "-- Database Name: ".$db->info("name")."\n";
			$header .= "-- Date: ".date("r")."\n";
			$header .= "-- Host: ".$_SERVER["HTTP_HOST"]."\n";
			$header .= "-- Generation time: ".substr(getmicrotime() - $backup_start, 0, 5)." seconds\n";
			$header .= "--\n\n";

			$buff = $header.$buff;

			$file .= ".sql";

			// compressing backup data
			switch ($comp) {
				case "bz2":
					header("Content-disposition: filename=".$file.".bz2");
					header("Content-type: application/x-bzip2");
					$buff = bzcompress($buff);
				break;
				case "gzip":
					header("Content-disposition: filename=".$file.".gz");
					header("Content-type: application/x-gzip");
					$buff = gzencode($buff);
				break;
				case "sql":
					header("Content-disposition: filename=".$file);
					header("Content-type: text/plain");
				break;
			}

			header("Content-Length: ".strlen($buff)."");
			header("Pragma: no_cache");
			header("Expires: 0");

			// output buffer
			echo $buff;

			slog("backup: backup", "actions", 1);

			die;

		break;
		case "restore":
			require_once GEKKO_ROOT."lib/file.lib.php";

			gpGlobalize("file, comp");

			$restore_start = getmicrotime();

			// getting submitted file
			$thumb = file_get("file", "", "temp");

			// no filetype was specified
			if (!$comp) {
				// guessing filetype by identifing header (snipped borrowed from phpMyAdmin)
				$fh = fopen($thumb["abs_loc"], "r");
				$hd = fread($fh, 3);
				fclose($fh);

				if ($hd == "BZh") {
					$comp = "bzip2";
				} elseif ($hd[0] == chr(31) && $hd[1] == chr(139)) {
					$comp = "gzip";
				}
			}

			// reading file into buffer
			$buff = "";
			switch ($comp) {
				case "bzip2":
					$fh = fopen($thumb["abs_loc"], "r");
					$buff = bzdecompress(fread($fh, filesize($thumb["abs_loc"])));
					fclose($fh);
				break;
				case "gzip":
					$fh = gzopen($thumb["abs_loc"], "rb");
					while (!gzeof($fh)) {
						$buff .= gzgetc($fh);
					}
					gzclose($fh);
				break;
				default:
					$fh = fopen($thumb["abs_loc"], "r");
					$buff = fread($fh, filesize($thumb["abs_loc"]));
					fclose($fh);
				break;
			}

			// deleting temporary file
			unlink($thumb["abs_loc"]);

			$buff .= trim($buff)."\n";

			$i = 0;
			// parseing queries
			while ($buff) {
				if (($eol = strpos($buff, ";\n")) !== false) {
					// getting line
					$query = substr($buff, 0, $eol+2);
					// stripping line from buffer
					$buff = substr($buff, $eol+2);
				} else {
					// this seems to be EOF
					$query = $buff;
					$buff = "";
				}
				// removing commets and blank lines from query
				$check = explode("\n", $query);
				// setting query as null again
				$query = "";
				foreach ($check as $c) {
					// ignoring comments (files that begin with "--" or "#") and blank lines
					if ($c && substr($c, 0, 1) != "#" && substr($c, 0, 2) != "--") {
						// appending this line to query (in case of multiple line queries)
						$query .= $c;
					}
				}
				if ($query) {
					// executing query
					$db->query($query, __FILE__, __LINE__);
					$i++;
				}
			}

			$time_taken = substr(getmicrotime() - $restore_start, 0, 5);

			$return .= "../../index.php/admin/backup?message=".sprintf(_l("L_DB_RESTORATION_SUCCESS"), $i, $time_taken)."";

		break;
		default:
			abort(_l("L_UNDEFINED_ACTION"));
		break;
	}

	redirect($return ? $return : $_SERVER["HTTP_REFERER"]);

	db_exit();
?>